import { findUISchema, getFirstPrimitiveProp, Resolve, composePaths, createDefaultValue, rankWith, schemaTypeIs, isDescriptionHidden, computeLabel, isStringControl, and, isMultiLineControl, isNumberControl, isIntegerControl, isEnumControl, isOneOfEnumControl, isDateControl, isDateTimeControl, isTimeControl, isBooleanControl, isLayout, uiTypeIs } from '@jsonforms/core';
import { inject, computed, ref, defineComponent, openBlock, createElementBlock, normalizeClass, createElementVNode, toDisplayString, renderSlot, resolveComponent, Fragment, renderList, createVNode, withCtx, createCommentVNode, createBlock, mergeProps } from 'vue';
import { DispatchRenderer, rendererProps, useJsonFormsArrayControl, useJsonFormsControl, useJsonFormsEnumControl, useJsonFormsOneOfEnumControl, useJsonFormsLayout, useJsonFormsLabel } from '@jsonforms/vue';
import merge from 'lodash/merge';
import cloneDeep from 'lodash/cloneDeep';
import mergeWith from 'lodash/mergeWith';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _taggedTemplateLiteral(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var defaultStyles = {
  control: {
    root: 'control',
    wrapper: 'wrapper',
    label: 'label',
    description: 'description',
    input: 'input',
    error: 'error',
    textarea: 'text-area',
    select: 'select',
    option: 'option'
  },
  verticalLayout: {
    root: 'vertical-layout',
    item: 'vertical-layout-item'
  },
  horizontalLayout: {
    root: 'horizontal-layout',
    item: 'horizontal-layout-item'
  },
  group: {
    root: 'group',
    label: 'group-label',
    item: 'group-item'
  },
  arrayList: {
    root: 'array-list',
    legend: 'array-list-legend',
    addButton: 'array-list-add',
    label: 'array-list-label',
    itemWrapper: 'array-list-item-wrapper',
    noData: 'array-list-no-data',
    item: 'array-list-item',
    itemToolbar: 'array-list-item-toolbar',
    itemLabel: 'array-list-item-label',
    itemContent: 'array-list-item-content',
    itemExpanded: 'expanded',
    itemMoveUp: 'array-list-item-move-up',
    itemMoveDown: 'array-list-item-move-down',
    itemDelete: 'array-list-item-delete'
  },
  label: {
    root: 'label-element'
  }
};

var createEmptyStyles = function createEmptyStyles() {
  return {
    control: {},
    verticalLayout: {},
    horizontalLayout: {},
    group: {},
    arrayList: {},
    label: {}
  };
};
var useStyles = function useStyles(element) {
  var _element$options, _element$options2;
  var userStyles = inject('styles', defaultStyles);
  if (!(element !== null && element !== void 0 && (_element$options = element.options) !== null && _element$options !== void 0 && _element$options.styles)) {
    return userStyles;
  }
  var styles = createEmptyStyles();
  if (userStyles) {
    merge(styles, userStyles);
  } else {
    merge(styles, defaultStyles);
  }
  if (element !== null && element !== void 0 && (_element$options2 = element.options) !== null && _element$options2 !== void 0 && _element$options2.styles) {
    merge(styles, element.options.styles);
  }
  return styles;
};

var classes = function classes(strings) {
  for (var _len = arguments.length, variables = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    variables[_key - 1] = arguments[_key];
  }
  return strings.reduce(function (acc, curr, index) {
    return "".concat(acc).concat(curr).concat(variables[index] || '');
  }, '').trim();
};
var mergeStyles = function mergeStyles(stylesA, stylesB) {
  var styles = cloneDeep(stylesA);
  mergeWith(styles, stylesB, function (aValue, bValue) {
    if (typeof aValue === 'string' && typeof bValue === 'string') {
      return "".concat(aValue, " ").concat(bValue);
    }
    return undefined;
  });
  return styles;
};

var useVanillaControl = function useVanillaControl(input) {
  var adaptTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (v) {
    return v.value;
  };
  var appliedOptions = computed(function () {
    return merge({}, cloneDeep(input.control.value.config), cloneDeep(input.control.value.uischema.options));
  });
  var isFocused = ref(false);
  var onChange = function onChange(event) {
    input.handleChange(input.control.value.path, adaptTarget(event.target));
  };
  var controlWrapper = computed(function () {
    var _input$control$value = input.control.value,
        id = _input$control$value.id,
        description = _input$control$value.description,
        errors = _input$control$value.errors,
        label = _input$control$value.label,
        visible = _input$control$value.visible,
        required = _input$control$value.required;
    return {
      id: id,
      description: description,
      errors: errors,
      label: label,
      visible: visible,
      required: required
    };
  });
  return _objectSpread2(_objectSpread2({}, input), {}, {
    styles: useStyles(input.control.value.uischema),
    isFocused: isFocused,
    appliedOptions: appliedOptions,
    controlWrapper: controlWrapper,
    onChange: onChange
  });
};
var useVanillaLayout = function useVanillaLayout(input) {
  var appliedOptions = computed(function () {
    return merge({}, cloneDeep(input.layout.value.config), cloneDeep(input.layout.value.uischema.options));
  });
  return _objectSpread2(_objectSpread2({}, input), {}, {
    styles: useStyles(input.layout.value.uischema),
    appliedOptions: appliedOptions
  });
};
var useVanillaLabel = function useVanillaLabel(input) {
  var appliedOptions = computed(function () {
    return merge({}, cloneDeep(input.label.value.config), cloneDeep(input.label.value.uischema.options));
  });
  return _objectSpread2(_objectSpread2({}, input), {}, {
    styles: useStyles(input.label.value.uischema),
    appliedOptions: appliedOptions
  });
};
var useVanillaArrayControl = function useVanillaArrayControl(input) {
  var appliedOptions = computed(function () {
    return merge({}, cloneDeep(input.control.value.config), cloneDeep(input.control.value.uischema.options));
  });
  var childUiSchema = computed(function () {
    return findUISchema(input.control.value.uischemas, input.control.value.schema, input.control.value.uischema.scope, input.control.value.path, undefined, input.control.value.uischema, input.control.value.rootSchema);
  });
  var childLabelForIndex = function childLabelForIndex(index) {
    var _input$control$value$, _input$control$value$2;
    var childLabelProp = (_input$control$value$ = (_input$control$value$2 = input.control.value.uischema.options) === null || _input$control$value$2 === void 0 ? void 0 : _input$control$value$2.childLabelProp) !== null && _input$control$value$ !== void 0 ? _input$control$value$ : getFirstPrimitiveProp(input.control.value.schema);
    if (!childLabelProp) {
      return "".concat(index);
    }
    var labelValue = Resolve.data(input.control.value.data, composePaths("".concat(index), childLabelProp));
    if (labelValue === undefined || labelValue === null || labelValue === NaN) {
      return '';
    }
    return "".concat(labelValue);
  };
  return _objectSpread2(_objectSpread2({}, input), {}, {
    styles: useStyles(input.control.value.uischema),
    appliedOptions: appliedOptions,
    childUiSchema: childUiSchema,
    childLabelForIndex: childLabelForIndex
  });
};

var _templateObject, _templateObject2;
var listItem = defineComponent({
  name: 'array-list-element',
  props: {
    initiallyExpanded: {
      required: false,
      type: Boolean,
      "default": false
    },
    label: {
      required: false,
      type: String,
      "default": ''
    },
    moveUpEnabled: {
      required: false,
      type: Boolean,
      "default": true
    },
    moveDownEnabled: {
      required: false,
      type: Boolean,
      "default": true
    },
    moveUp: {
      required: false,
      type: Function,
      "default": undefined
    },
    moveDown: {
      required: false,
      type: Function,
      "default": undefined
    },
    "delete": {
      required: false,
      type: Function,
      "default": undefined
    },
    styles: {
      required: true,
      type: Object
    }
  },
  data: function data() {
    return {
      expanded: this.initiallyExpanded
    };
  },
  computed: {
    contentClasses: function contentClasses() {
      return classes(_templateObject || (_templateObject = _taggedTemplateLiteral(["", " ", ""])), this.styles.arrayList.itemContent, this.expanded && this.styles.arrayList.itemExpanded);
    },
    toolbarClasses: function toolbarClasses() {
      return classes(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral(["", " ", ""])), this.styles.arrayList.itemToolbar, this.expanded && this.styles.arrayList.itemExpanded);
    }
  },
  methods: {
    expandClicked: function expandClicked() {
      this.expanded = !this.expanded;
    },
    moveUpClicked: function moveUpClicked(event) {
      var _this$moveUp;
      event.stopPropagation();
      (_this$moveUp = this.moveUp) === null || _this$moveUp === void 0 ? void 0 : _this$moveUp.call(this);
    },
    moveDownClicked: function moveDownClicked(event) {
      var _this$moveDown;
      event.stopPropagation();
      (_this$moveDown = this.moveDown) === null || _this$moveDown === void 0 ? void 0 : _this$moveDown.call(this);
    },
    deleteClicked: function deleteClicked(event) {
      var _this$delete;
      event.stopPropagation();
      (_this$delete = this["delete"]) === null || _this$delete === void 0 ? void 0 : _this$delete.call(this);
    }
  }
});

var _hoisted_1$b = ["disabled"];
var _hoisted_2$3 = ["disabled"];
function render$f(_ctx, _cache, $props, $setup, $data, $options) {
  return openBlock(), createElementBlock("div", {
    "class": normalizeClass(_ctx.styles.arrayList.item)
  }, [createElementVNode("div", {
    onClick: _cache[3] || (_cache[3] = function () {
      return _ctx.expandClicked && _ctx.expandClicked.apply(_ctx, arguments);
    }),
    "class": normalizeClass(_ctx.toolbarClasses)
  }, [createElementVNode("div", {
    "class": normalizeClass(_ctx.styles.arrayList.itemLabel)
  }, toDisplayString(_ctx.label), 3), createElementVNode("button", {
    onClick: _cache[0] || (_cache[0] = function () {
      return _ctx.moveUpClicked && _ctx.moveUpClicked.apply(_ctx, arguments);
    }),
    disabled: !_ctx.moveUpEnabled,
    "class": normalizeClass(_ctx.styles.arrayList.itemMoveUp),
    type: "button"
  }, " ↑ ", 10, _hoisted_1$b), createElementVNode("button", {
    onClick: _cache[1] || (_cache[1] = function () {
      return _ctx.moveDownClicked && _ctx.moveDownClicked.apply(_ctx, arguments);
    }),
    disabled: !_ctx.moveDownEnabled,
    "class": normalizeClass(_ctx.styles.arrayList.itemMoveDown),
    type: "button"
  }, " ↓ ", 10, _hoisted_2$3), createElementVNode("button", {
    onClick: _cache[2] || (_cache[2] = function () {
      return _ctx.deleteClicked && _ctx.deleteClicked.apply(_ctx, arguments);
    }),
    "class": normalizeClass(_ctx.styles.arrayList.itemDelete),
    type: "button"
  }, " 🗙 ", 2)], 2), createElementVNode("div", {
    "class": normalizeClass(_ctx.contentClasses)
  }, [renderSlot(_ctx.$slots, "default")], 2)], 2);
}

listItem.render = render$f;

var controlRenderer$a = defineComponent({
  name: 'array-list-renderer',
  components: {
    ArrayListElement: listItem,
    DispatchRenderer: DispatchRenderer
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaArrayControl(useJsonFormsArrayControl(props));
  },
  computed: {
    noData: function noData() {
      return !this.control.data || this.control.data.length === 0;
    }
  },
  methods: {
    composePaths: composePaths,
    createDefaultValue: createDefaultValue,
    addButtonClick: function addButtonClick() {
      this.addItem(this.control.path, createDefaultValue(this.control.schema))();
    }
  }
});
var entry$d = {
  renderer: controlRenderer$a,
  tester: rankWith(2, schemaTypeIs('array'))
};

function render$e(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_dispatch_renderer = resolveComponent("dispatch-renderer");
  var _component_array_list_element = resolveComponent("array-list-element");
  return _ctx.control.visible ? (openBlock(), createElementBlock("fieldset", {
    key: 0,
    "class": normalizeClass(_ctx.styles.arrayList.root)
  }, [createElementVNode("legend", {
    "class": normalizeClass(_ctx.styles.arrayList.legend)
  }, [createElementVNode("button", {
    "class": normalizeClass(_ctx.styles.arrayList.addButton),
    onClick: _cache[0] || (_cache[0] = function () {
      return _ctx.addButtonClick && _ctx.addButtonClick.apply(_ctx, arguments);
    }),
    type: "button"
  }, " + ", 2), createElementVNode("label", {
    "class": normalizeClass(_ctx.styles.arrayList.label)
  }, toDisplayString(_ctx.control.label), 3)], 2), (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.control.data, function (element, index) {
    return openBlock(), createElementBlock("div", {
      key: "".concat(_ctx.control.path, "-").concat(index),
      "class": normalizeClass(_ctx.styles.arrayList.itemWrapper)
    }, [createVNode(_component_array_list_element, {
      moveUp: _ctx.moveUp(_ctx.control.path, index),
      moveUpEnabled: index > 0,
      moveDown: _ctx.moveDown(_ctx.control.path, index),
      moveDownEnabled: index < _ctx.control.data.length - 1,
      "delete": _ctx.removeItems(_ctx.control.path, [index]),
      label: _ctx.childLabelForIndex(index),
      styles: _ctx.styles
    }, {
      "default": withCtx(function () {
        return [createVNode(_component_dispatch_renderer, {
          schema: _ctx.control.schema,
          uischema: _ctx.childUiSchema,
          path: _ctx.composePaths(_ctx.control.path, "".concat(index)),
          enabled: _ctx.control.enabled,
          renderers: _ctx.control.renderers,
          cells: _ctx.control.cells
        }, null, 8, ["schema", "uischema", "path", "enabled", "renderers", "cells"])];
      }),
      _: 2
    }, 1032, ["moveUp", "moveUpEnabled", "moveDown", "moveDownEnabled", "delete", "label", "styles"])], 2);
  }), 128)), _ctx.noData ? (openBlock(), createElementBlock("div", {
    key: 0,
    "class": normalizeClass(_ctx.styles.arrayList.noData)
  }, " No data ", 2)) : createCommentVNode("", true)], 2)) : createCommentVNode("", true);
}

controlRenderer$a.render = render$e;

var arrayRenderers = [entry$d];

var script = defineComponent({
  name: 'control-wrapper',
  props: {
    id: {
      required: true,
      type: String
    },
    description: {
      required: false,
      type: String,
      "default": undefined
    },
    errors: {
      required: false,
      type: String,
      "default": undefined
    },
    label: {
      required: false,
      type: String,
      "default": undefined
    },
    appliedOptions: {
      required: false,
      type: Object,
      "default": undefined
    },
    visible: {
      required: false,
      type: Boolean,
      "default": true
    },
    required: {
      required: false,
      type: Boolean,
      "default": false
    },
    isFocused: {
      required: false,
      type: Boolean,
      "default": false
    },
    styles: {
      required: true,
      type: Object
    }
  },
  computed: {
    showDescription: function showDescription() {
      var _this$appliedOptions;
      return !isDescriptionHidden(this.visible, this.description, this.isFocused, !!((_this$appliedOptions = this.appliedOptions) !== null && _this$appliedOptions !== void 0 && _this$appliedOptions.showUnfocusedDescription));
    },
    computedLabel: function computedLabel() {
      var _this$appliedOptions2;
      return computeLabel(this.label, this.required, !!((_this$appliedOptions2 = this.appliedOptions) !== null && _this$appliedOptions2 !== void 0 && _this$appliedOptions2.hideRequiredAsterisk));
    }
  }
});

var _hoisted_1$a = ["id"];
var _hoisted_2$2 = ["for"];
function render$d(_ctx, _cache, $props, $setup, $data, $options) {
  return _ctx.visible ? (openBlock(), createElementBlock("div", {
    key: 0,
    "class": normalizeClass(_ctx.styles.control.root),
    id: _ctx.id
  }, [createElementVNode("label", {
    "for": _ctx.id + '-input',
    "class": normalizeClass(_ctx.styles.control.label)
  }, toDisplayString(_ctx.computedLabel), 11, _hoisted_2$2), createElementVNode("div", {
    "class": normalizeClass(_ctx.styles.control.wrapper)
  }, [renderSlot(_ctx.$slots, "default")], 2), createElementVNode("div", {
    "class": normalizeClass(_ctx.errors ? _ctx.styles.control.error : _ctx.styles.control.description)
  }, toDisplayString(_ctx.errors ? _ctx.errors : _ctx.showDescription ? _ctx.description : null), 3)], 10, _hoisted_1$a)) : createCommentVNode("", true);
}

script.render = render$d;

var controlRenderer$9 = defineComponent({
  name: 'string-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value || undefined;
    });
  }
});
var entry$c = {
  renderer: controlRenderer$9,
  tester: rankWith(1, isStringControl)
};

var _hoisted_1$9 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$c(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$9)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$9.render = render$c;

var controlRenderer$8 = defineComponent({
  name: 'multi-string-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value || undefined;
    });
  }
});
var entry$b = {
  renderer: controlRenderer$8,
  tester: rankWith(2, and(isStringControl, isMultiLineControl))
};

var _hoisted_1$8 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$b(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("textarea", {
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.textarea),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$8)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$8.render = render$b;

var controlRenderer$7 = defineComponent({
  name: 'number-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value === '' ? undefined : Number(target.value);
    });
  },
  computed: {
    step: function step() {
      var _options$step;
      var options = this.appliedOptions;
      return (_options$step = options.step) !== null && _options$step !== void 0 ? _options$step : 0.1;
    }
  }
});
var entry$a = {
  renderer: controlRenderer$7,
  tester: rankWith(1, isNumberControl)
};

var _hoisted_1$7 = ["step", "id", "value", "disabled", "autofocus", "placeholder"];
function render$a(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "number",
        step: _ctx.step,
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$7)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$7.render = render$a;

var controlRenderer$6 = defineComponent({
  name: 'integer-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value === '' ? undefined : parseInt(target.value, 10);
    });
  }
});
var entry$9 = {
  renderer: controlRenderer$6,
  tester: rankWith(1, isIntegerControl)
};

var _hoisted_1$6 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$9(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "number",
        step: 1,
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$6)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$6.render = render$9;

var controlRenderer$5 = defineComponent({
  name: 'enum-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsEnumControl(props), function (target) {
      return target.selectedIndex === 0 ? undefined : target.value;
    });
  }
});
var entry$8 = {
  renderer: controlRenderer$5,
  tester: rankWith(2, isEnumControl)
};

var _hoisted_1$5 = ["id", "value", "disabled", "autofocus"];
var _hoisted_2$1 = ["value", "label"];
function render$8(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("select", {
        id: _ctx.control.id + '-select',
        "class": normalizeClass(_ctx.styles.control.select),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, [createElementVNode("option", {
        value: "",
        key: "empty",
        "class": normalizeClass(_ctx.styles.control.option)
      }, null, 2), (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.control.options, function (optionElement) {
        return openBlock(), createElementBlock("option", {
          key: optionElement.value,
          value: optionElement.value,
          label: optionElement.label,
          "class": normalizeClass(_ctx.styles.control.option)
        }, null, 10, _hoisted_2$1);
      }), 128))], 42, _hoisted_1$5)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$5.render = render$8;

var controlRenderer$4 = defineComponent({
  name: 'enum-oneof-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsOneOfEnumControl(props), function (target) {
      return target.selectedIndex === 0 ? undefined : target.value;
    });
  }
});
var entry$7 = {
  renderer: controlRenderer$4,
  tester: rankWith(2, isOneOfEnumControl)
};

var _hoisted_1$4 = ["id", "value", "disabled", "autofocus"];
var _hoisted_2 = ["value", "label"];
function render$7(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("select", {
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.select),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, [createElementVNode("option", {
        value: "",
        key: "empty",
        "class": normalizeClass(_ctx.styles.control.option)
      }, null, 2), (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.control.options, function (optionElement) {
        return openBlock(), createElementBlock("option", {
          key: optionElement.value,
          value: optionElement.value,
          label: optionElement.label,
          "class": normalizeClass(_ctx.styles.control.option)
        }, null, 10, _hoisted_2);
      }), 128))], 42, _hoisted_1$4)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$4.render = render$7;

var controlRenderer$3 = defineComponent({
  name: 'date-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value || undefined;
    });
  }
});
var entry$6 = {
  renderer: controlRenderer$3,
  tester: rankWith(2, isDateControl)
};

var _hoisted_1$3 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$6(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "date",
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$3)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$3.render = render$6;

var toISOString = function toISOString(inputDateTime) {
  return inputDateTime === '' ? undefined : inputDateTime + ':00.000Z';
};
var controlRenderer$2 = defineComponent({
  name: 'datetime-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return toISOString(target.value);
    });
  },
  computed: {
    dataTime: function dataTime() {
      var _this$control$data;
      return ((_this$control$data = this.control.data) !== null && _this$control$data !== void 0 ? _this$control$data : '').substr(0, 16);
    }
  }
});
var entry$5 = {
  renderer: controlRenderer$2,
  tester: rankWith(2, isDateTimeControl)
};

var _hoisted_1$2 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$5(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "datetime-local",
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.dataTime,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$2)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$2.render = render$5;

var controlRenderer$1 = defineComponent({
  name: 'time-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.value || undefined;
    });
  }
});
var entry$4 = {
  renderer: controlRenderer$1,
  tester: rankWith(2, isTimeControl)
};

var _hoisted_1$1 = ["id", "value", "disabled", "autofocus", "placeholder"];
function render$4(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "time",
        id: _ctx.control.id + '-input',
        "class": normalizeClass(_ctx.styles.control.input),
        value: _ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1$1)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer$1.render = render$4;

var controlRenderer = defineComponent({
  name: 'boolean-control-renderer',
  components: {
    ControlWrapper: script
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaControl(useJsonFormsControl(props), function (target) {
      return target.checked;
    });
  }
});
var entry$3 = {
  renderer: controlRenderer,
  tester: rankWith(1, isBooleanControl)
};

var _hoisted_1 = ["id", "checked", "disabled", "autofocus", "placeholder"];
function render$3(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_control_wrapper = resolveComponent("control-wrapper");
  return openBlock(), createBlock(_component_control_wrapper, mergeProps(_ctx.controlWrapper, {
    styles: _ctx.styles,
    isFocused: _ctx.isFocused,
    appliedOptions: _ctx.appliedOptions
  }), {
    "default": withCtx(function () {
      return [createElementVNode("input", {
        type: "checkbox",
        "class": normalizeClass(_ctx.styles.control.input),
        id: _ctx.control.id + '-input',
        checked: !!_ctx.control.data,
        disabled: !_ctx.control.enabled,
        autofocus: _ctx.appliedOptions.focus,
        placeholder: _ctx.appliedOptions.placeholder,
        onChange: _cache[0] || (_cache[0] = function () {
          return _ctx.onChange && _ctx.onChange.apply(_ctx, arguments);
        }),
        onFocus: _cache[1] || (_cache[1] = function ($event) {
          return _ctx.isFocused = true;
        }),
        onBlur: _cache[2] || (_cache[2] = function ($event) {
          return _ctx.isFocused = false;
        })
      }, null, 42, _hoisted_1)];
    }),
    _: 1
  }, 16, ["styles", "isFocused", "appliedOptions"]);
}

controlRenderer.render = render$3;

var controlRenderers = [entry$c, entry$b, entry$a, entry$9, entry$8, entry$7, entry$6, entry$5, entry$4, entry$3];

var layoutRenderer$1 = defineComponent({
  name: 'layout-renderer',
  components: {
    DispatchRenderer: DispatchRenderer
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaLayout(useJsonFormsLayout(props));
  },
  computed: {
    layoutClassObject: function layoutClassObject() {
      return this.layout.direction === 'row' ? this.styles.horizontalLayout : this.styles.verticalLayout;
    }
  }
});
var entry$2 = {
  renderer: layoutRenderer$1,
  tester: rankWith(1, isLayout)
};

function render$2(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_dispatch_renderer = resolveComponent("dispatch-renderer");
  return _ctx.layout.visible ? (openBlock(), createElementBlock("div", {
    key: 0,
    "class": normalizeClass(_ctx.layoutClassObject.root)
  }, [(openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.layout.uischema.elements, function (element, index) {
    return openBlock(), createElementBlock("div", {
      key: "".concat(_ctx.layout.path, "-").concat(index),
      "class": normalizeClass(_ctx.layoutClassObject.item)
    }, [createVNode(_component_dispatch_renderer, {
      schema: _ctx.layout.schema,
      uischema: element,
      path: _ctx.layout.path,
      enabled: _ctx.layout.enabled,
      renderers: _ctx.layout.renderers,
      cells: _ctx.layout.cells
    }, null, 8, ["schema", "uischema", "path", "enabled", "renderers", "cells"])], 2);
  }), 128))], 2)) : createCommentVNode("", true);
}

layoutRenderer$1.render = render$2;

var layoutRenderer = defineComponent({
  name: 'group-renderer',
  components: {
    DispatchRenderer: DispatchRenderer
  },
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaLayout(useJsonFormsLayout(props));
  }
});
var entry$1 = {
  renderer: layoutRenderer,
  tester: rankWith(2, and(isLayout, uiTypeIs('Group')))
};

function render$1(_ctx, _cache, $props, $setup, $data, $options) {
  var _component_dispatch_renderer = resolveComponent("dispatch-renderer");
  return _ctx.layout.visible ? (openBlock(), createElementBlock("fieldset", {
    key: 0,
    "class": normalizeClass(_ctx.styles.group.root)
  }, [_ctx.layout.label ? (openBlock(), createElementBlock("legend", {
    key: 0,
    "class": normalizeClass(_ctx.styles.group.label)
  }, toDisplayString(_ctx.layout.label), 3)) : createCommentVNode("", true), (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.layout.uischema.elements, function (element, index) {
    return openBlock(), createElementBlock("div", {
      key: "".concat(_ctx.layout.path, "-").concat(index),
      "class": normalizeClass(_ctx.styles.group.item)
    }, [createVNode(_component_dispatch_renderer, {
      schema: _ctx.layout.schema,
      uischema: element,
      path: _ctx.layout.path,
      enabled: _ctx.layout.enabled,
      renderers: _ctx.layout.renderers,
      cells: _ctx.layout.cells
    }, null, 8, ["schema", "uischema", "path", "enabled", "renderers", "cells"])], 2);
  }), 128))], 2)) : createCommentVNode("", true);
}

layoutRenderer.render = render$1;

var layoutRenderers = [entry$2, entry$1];

var labelRenderer = defineComponent({
  name: 'label-renderer',
  props: _objectSpread2({}, rendererProps()),
  setup: function setup(props) {
    return useVanillaLabel(useJsonFormsLabel(props));
  }
});
var entry = {
  renderer: labelRenderer,
  tester: rankWith(1, uiTypeIs('Label'))
};

function render(_ctx, _cache, $props, $setup, $data, $options) {
  return _ctx.label.visible ? (openBlock(), createElementBlock("label", {
    key: 0,
    "class": normalizeClass(_ctx.styles.label.root)
  }, toDisplayString(_ctx.label.text), 3)) : createCommentVNode("", true);
}

labelRenderer.render = render;

var labelRenderers = [entry];

var vanillaRenderers = [].concat(_toConsumableArray(controlRenderers), _toConsumableArray(layoutRenderers), _toConsumableArray(arrayRenderers), _toConsumableArray(labelRenderers));

export { controlRenderer$a as ArrayListRenderer, controlRenderer as BooleanControlRenderer, script as ControlWrapper, controlRenderer$3 as DateControlRenderer, controlRenderer$2 as DateTimeControlRenderer, controlRenderer$5 as EnumControlRenderer, layoutRenderer as GroupRenderer, controlRenderer$6 as IntegerControlRenderer, labelRenderer as LabelRenderer, layoutRenderer$1 as LayoutRenderer, controlRenderer$8 as MultiStringControlRenderer, controlRenderer$7 as NumberControlRenderer, controlRenderer$9 as StringControlRenderer, controlRenderer$1 as TimeControlRenderer, arrayRenderers, classes, controlRenderers, defaultStyles, labelRenderers, layoutRenderers, mergeStyles, controlRenderer$4 as oneOfEnumControlRenderer, useStyles, useVanillaArrayControl, useVanillaControl, useVanillaLabel, useVanillaLayout, vanillaRenderers };
//# sourceMappingURL=jsonforms-vue-vanilla.esm.js.map
